==> packagesearch_2.2.5.tar.gz: packagesearch-2.2.5/src/plugins/debtagsplugin/debtagsplugincontainer.cpp <==
#include <cassert>

#include <qmessagebox.h>
#include <qaction.h>
#include <q3popupmenu.h>
#include <QMainWindow>
#include <QMutex>

#include <unistd.h>		// for getuid and geteuid
#include <sys/types.h>	//

#include <wibble/operators.h>
#include <ept/cache/apt/packages.h>
#include <ept/cache/debtags/tagmap.h>
#include <ept/cache/debtags/vocabulary.h>

#include "debtagsplugincontainer.h"

// NUtil
#include "helpers.h"

// NApplication
#include "applicationfactory.h"
#include "runcommand.h"

// NPlugin
#include <iprovider.h>
#include <plugincontainer.h>
#include <iprogressobserver.h>
#include "debtagsplugin.h"
#include "relatedplugin.h"
#include "debtagspluginfactory.h"

#include "debtagssettingswidget.h"

extern "C" 
{ 
	NPlugin::PluginContainer* new_debtagsplugin() 
	{
		return new NPlugin::DebtagsPluginContainer;
	} 

	NPlugin::PluginInformation get_pluginInformation()
	{
		return NPlugin::PluginInformation("debtagsplugin", "2.2.5", "Benjamin Mesing");
	} 
}

/** Initialize the plugin. */
__attribute__ ((constructor)) void init() 
{
}
      
      
// __attribute__ ((destructor)) void fini() 
// {
//   /* code here is executed just before dlclose() unloads the module */
// } 


namespace NPlugin
{

DebtagsPluginContainer::DebtagsPluginContainer()
{
	// this assumes, that only one instance of the DebtagsPluginContainer is created
	// (no two versions of libapt-front may be opened at any time)
	// This is ok for our purpose - though usually there should be a singleton ensuring
	// this constraint...
	DebtagsPluginFactory::getInstance()->setContainer(this);
	_pCommand = 0;
	_pRelatedPlugin = 0;
	_pDebtagsPlugin = 0;
	_pSettingsWidget = 0;
	addPlugin("DebtagsPlugin");
	addPlugin("RelatedPlugin");
	
	_debtagsEnabled=false;
	
}
 
DebtagsPluginContainer::~DebtagsPluginContainer()
{
	delete _pCommand;
//	qDebug("Closing global aptFront Cache");
//	cache::Global::get().close();
}

/////////////////////////////////////////////////////
// PluginContainer Interface
/////////////////////////////////////////////////////

bool DebtagsPluginContainer::init(IProvider* pProvider)
{
	BasePluginContainer::init(pProvider, DebtagsPluginFactory::getInstance());
	updateDebtags();
	if (debtagsEnabled())
	{
		// use dynamic cast here because of the virtual base class 
		// (static_cast is not allowed there)
		_pRelatedPlugin = dynamic_cast<RelatedPlugin*>(requestPlugin("RelatedPlugin"));
		_pDebtagsPlugin = dynamic_cast<DebtagsPlugin*>(requestPlugin("DebtagsPlugin"));
	}
	return debtagsEnabled();
}

vector< pair<QString, QAction*> > DebtagsPluginContainer::actions()
{
	vector< pair<QString, QAction*> > result;
	return result;
}

QWidget* DebtagsPluginContainer::getSettingsWidget(QWidget* pParent)
{
	_pSettingsWidget = new DebtagsSettingsWidget(_hiddenFacets, pParent, "DebtagsSettingsWidget");
 	return _pSettingsWidget;
}

void DebtagsPluginContainer::applySettings()
{
	assert(_pSettingsWidget);
	_hiddenFacets = _pSettingsWidget->hiddenFacets();
	updateVocabulary();
}



/////////////////////////////////////////////////////
// BasePluginContainer Interface
/////////////////////////////////////////////////////


QDomElement DebtagsPluginContainer::loadContainerSettings(const QDomElement source)
{
	if (source.tagName() != "ContainerSettings")
		return source;
	float settingsVersion;
	NXml::getAttribute(source, settingsVersion, "settingsVersion", 0.0f);
	
	QDomNodeList hiddenFacets = source.elementsByTagName("HiddenFacet");
	for (int i=0; i < hiddenFacets.count(); ++i)
	{
		string hiddenFacet = toString( hiddenFacets.item(i).toElement().text() );
		_hiddenFacets.insert(hiddenFacet);
	}
	if (debtagsEnabled())
		updateVocabulary();
	return NXml::getNextElement(source);
}

void DebtagsPluginContainer::saveContainerSettings(NXml::XmlData& outData, QDomElement parent) const
{
	QDomElement containerElement = outData.addElement(parent, "ContainerSettings");
	outData.addAttribute(containerElement, 0.1f, "settingsVersion");
	for (set<string>::const_iterator it = _hiddenFacets.begin(); it != _hiddenFacets.end(); ++it)
	{
		QDomElement hiddenFacetElement = outData.addElement(containerElement, "HiddenFacet");
		outData.addText(hiddenFacetElement, *it);
	}
}





/////////////////////////////////////////////////////
// Helper Methods
/////////////////////////////////////////////////////

void DebtagsPluginContainer::updateVocabulary(bool informPlugins)
{
	if (informPlugins)
	{
		if (_pDebtagsPlugin)
			_pDebtagsPlugin->debtagsDataChanged();
		if (_pRelatedPlugin)
			_pRelatedPlugin->debtagsDataChanged();
	}
}

void DebtagsPluginContainer::updateDebtags()
{
	NUtil::IProgressObserver* pProgressObserver = provider()->progressObserver();
	if (pProgressObserver)
		pProgressObserver->setText("Loading Debtags Plugin");
/*	try 
	{
		// this is set to 0 if no listener is available
		cache::Global::get().close();
		qDebug("Opening aptfront cache for debtags");
		cache::Global::get().open(cache::Cache::OpenDefault
				| cache::Cache::OpenReadOnly
				| cache::Cache::OpenTags
				| cache::Cache::OpenDebtags);
	}
	catch (Tagcoll::SystemException e)*/
	
	if (!ept::configuration::apt::cache::TagMap::hasTagDatabase())
	{
		setDebtagsEnabled(false);	// disable the debtags system
		provider()->reportError(
			tr("Tag Database Not Available" ),
			tr(
				"<p>The tag database is not available and the debtags plugin was disabled!</p>"
				"<p>"
				"You must execute <tt><b>debtags update</b></tt> as root on the commandline to download "
				"the database. If debtags is not on your system you can install it via "
				"<tt><b>apt-get install debtags</b></tt><br>"
				"Afterwards you can enable the debtags plugin via the plugin menu -> Control Plugins."
				"</p>")
/*				"<p>"
				"The original error message reported by debtags was:<br>"
				"<tt>"
			) +
			toQString(e.desc()) +
			"</tt>"
			"</p>"*/
		);
		return;
	}
/*	catch (exception::Error e)
	{
		setDebtagsEnabled(false);	// disable the debtags system
		provider()->reportError(
			tr("Unknown Error when reading the Debtags database" ),
			tr("An unknown error occured when reading the debtags database.<br>"
				"The error reported by debtags is:.<br><tt>") +
			toQString(e.what()) +
			"</tt>"
		);
		return;
	}*/

	setDebtagsEnabled(true);
	if (debtagsEnabled())
	{
		// load the new vocabulary
		updateVocabulary(false);
	}
	// inform the plugins about the new collections
	if (_pDebtagsPlugin)
		_pDebtagsPlugin->debtagsDataChanged();
	if (_pRelatedPlugin)
		_pRelatedPlugin->debtagsDataChanged();
	if (pProgressObserver)
		pProgressObserver->setProgress(100);
}

void DebtagsPluginContainer::setDebtagsEnabled(bool enabled)
{
	_debtagsEnabled = enabled;
}

ept::configuration::apt::cache::TagMap* DebtagsPluginContainer::collection() const	
{
	return &(provider()->aptFrontCache().tagMap());
}

const std::set<DebtagsPluginContainer::Facet> DebtagsPluginContainer::facets() const
{
	using namespace wibble::operators;

	Vocabulary& tags = provider()->aptFrontCache().vocabulary();
	std::set<Facet> hidden;
	for (set<string>::const_iterator i = _hiddenFacets.begin();
		i != _hiddenFacets.end(); i++)
	{
		Facet f = tags.facetByName(*i);
		if (f.valid())
			hidden |= f;
	}
	return tags.facets() - hidden; 
}




}	// namespace NPlugin

#undef emit
#include <ept/cache/debtags/tagmap.tcc>
#include <ept/cache/debtags/vocabulary.tcc>
#include <ept/cache/apt/index.tcc>

==> packagesearch_2.2.6.tar.gz: packagesearch-2.2.6/src/plugins/debtagsplugin/debtagsplugincontainer.cpp <==
#include <cassert>

#include <qmessagebox.h>
#include <qaction.h>
#include <q3popupmenu.h>
#include <QMainWindow>
#include <QMutex>

#include <unistd.h>		// for getuid and geteuid
#include <sys/types.h>	//

#include <wibble/operators.h>
#include "aggregator.h"

#include "debtagsplugincontainer.h"

// NUtil
#include "helpers.h"

// NApplication
#include "applicationfactory.h"
#include "runcommand.h"

// NPlugin
#include <iprovider.h>
#include <plugincontainer.h>
#include <iprogressobserver.h>
#include "debtagsplugin.h"
#include "relatedplugin.h"
#include "debtagspluginfactory.h"

#include "debtagssettingswidget.h"

extern "C" 
{ 
	NPlugin::PluginContainer* new_debtagsplugin() 
	{
		return new NPlugin::DebtagsPluginContainer;
	} 

	NPlugin::PluginInformation get_pluginInformation()
	{
		return NPlugin::PluginInformation("debtagsplugin", "2.2.6", "Benjamin Mesing");
	} 
}

/** Initialize the plugin. */
__attribute__ ((constructor)) void init() 
{
}
      
      
// __attribute__ ((destructor)) void fini() 
// {
//   /* code here is executed just before dlclose() unloads the module */
// } 


namespace NPlugin
{

DebtagsPluginContainer::DebtagsPluginContainer()
{
	// this assumes, that only one instance of the DebtagsPluginContainer is created
	// (no two versions of libapt-front may be opened at any time)
	// This is ok for our purpose - though usually there should be a singleton ensuring
	// this constraint...
	DebtagsPluginFactory::getInstance()->setContainer(this);
	_pCommand = 0;
	_pRelatedPlugin = 0;
	_pDebtagsPlugin = 0;
	_pSettingsWidget = 0;
	addPlugin("DebtagsPlugin");
	addPlugin("RelatedPlugin");
	
	_debtagsEnabled=false;
	
}
 
DebtagsPluginContainer::~DebtagsPluginContainer()
{
	delete _pCommand;
//	qDebug("Closing global aptFront Cache");
//	cache::Global::get().close();
}

/////////////////////////////////////////////////////
// PluginContainer Interface
/////////////////////////////////////////////////////

bool DebtagsPluginContainer::init(IProvider* pProvider)
{
	BasePluginContainer::init(pProvider, DebtagsPluginFactory::getInstance());
	updateDebtags();
	if (debtagsEnabled())
	{
		// use dynamic cast here because of the virtual base class 
		// (static_cast is not allowed there)
		_pRelatedPlugin = dynamic_cast<RelatedPlugin*>(requestPlugin("RelatedPlugin"));
		_pDebtagsPlugin = dynamic_cast<DebtagsPlugin*>(requestPlugin("DebtagsPlugin"));
	}
	return debtagsEnabled();
}

vector< pair<QString, QAction*> > DebtagsPluginContainer::actions()
{
	vector< pair<QString, QAction*> > result;
	return result;
}

QWidget* DebtagsPluginContainer::getSettingsWidget(QWidget* pParent)
{
	_pSettingsWidget = new DebtagsSettingsWidget(_hiddenFacets, pParent, "DebtagsSettingsWidget");
 	return _pSettingsWidget;
}

void DebtagsPluginContainer::applySettings()
{
	assert(_pSettingsWidget);
	_hiddenFacets = _pSettingsWidget->hiddenFacets();
	updateVocabulary();
}



/////////////////////////////////////////////////////
// BasePluginContainer Interface
/////////////////////////////////////////////////////


QDomElement DebtagsPluginContainer::loadContainerSettings(const QDomElement source)
{
	if (source.tagName() != "ContainerSettings")
		return source;
	float settingsVersion;
	NXml::getAttribute(source, settingsVersion, "settingsVersion", 0.0f);
	
	QDomNodeList hiddenFacets = source.elementsByTagName("HiddenFacet");
	for (int i=0; i < hiddenFacets.count(); ++i)
	{
		string hiddenFacet = toString( hiddenFacets.item(i).toElement().text() );
		_hiddenFacets.insert(hiddenFacet);
	}
	if (debtagsEnabled())
		updateVocabulary();
	return NXml::getNextElement(source);
}

void DebtagsPluginContainer::saveContainerSettings(NXml::XmlData& outData, QDomElement parent) const
{
	QDomElement containerElement = outData.addElement(parent, "ContainerSettings");
	outData.addAttribute(containerElement, 0.1f, "settingsVersion");
	for (set<string>::const_iterator it = _hiddenFacets.begin(); it != _hiddenFacets.end(); ++it)
	{
		QDomElement hiddenFacetElement = outData.addElement(containerElement, "HiddenFacet");
		outData.addText(hiddenFacetElement, *it);
	}
}





/////////////////////////////////////////////////////
// Helper Methods
/////////////////////////////////////////////////////

void DebtagsPluginContainer::updateVocabulary(bool informPlugins)
{
	if (informPlugins)
	{
		if (_pDebtagsPlugin)
			_pDebtagsPlugin->debtagsDataChanged();
		if (_pRelatedPlugin)
			_pRelatedPlugin->debtagsDataChanged();
	}
}

void DebtagsPluginContainer::updateDebtags()
{
	NUtil::IProgressObserver* pProgressObserver = provider()->progressObserver();
	if (pProgressObserver)
		pProgressObserver->setText("Loading Debtags Plugin");
/*	try 
	{
		// this is set to 0 if no listener is available
		cache::Global::get().close();
		qDebug("Opening aptfront cache for debtags");
		cache::Global::get().open(cache::Cache::OpenDefault
				| cache::Cache::OpenReadOnly
				| cache::Cache::OpenTags
				| cache::Cache::OpenDebtags);
	}
	catch (Tagcoll::SystemException e)*/
	
	if (!collection()->hasData())
	{
		setDebtagsEnabled(false);	// disable the debtags system
		provider()->reportError(
			tr("Tag Database Not Available" ),
			tr(
				"<p>The tag database is not available and the debtags plugin was disabled!</p>"
				"<p>"
				"You must execute <tt><b>debtags update</b></tt> as root on the commandline to download "
				"the database. If debtags is not on your system you can install it via "
				"<tt><b>apt-get install debtags</b></tt><br>"
				"Afterwards you can enable the debtags plugin via the plugin menu -> Control Plugins."
				"</p>")
/*				"<p>"
				"The original error message reported by debtags was:<br>"
				"<tt>"
			) +
			toQString(e.desc()) +
			"</tt>"
			"</p>"*/
		);
		return;
	}
/*	catch (exception::Error e)
	{
		setDebtagsEnabled(false);	// disable the debtags system
		provider()->reportError(
			tr("Unknown Error when reading the Debtags database" ),
			tr("An unknown error occured when reading the debtags database.<br>"
				"The error reported by debtags is:.<br><tt>") +
			toQString(e.what()) +
			"</tt>"
		);
		return;
	}*/

	setDebtagsEnabled(true);
	if (debtagsEnabled())
	{
		// load the new vocabulary
		updateVocabulary(false);
	}
	// inform the plugins about the new collections
	if (_pDebtagsPlugin)
		_pDebtagsPlugin->debtagsDataChanged();
	if (_pRelatedPlugin)
		_pRelatedPlugin->debtagsDataChanged();
	if (pProgressObserver)
		pProgressObserver->setProgress(100);
}

void DebtagsPluginContainer::setDebtagsEnabled(bool enabled)
{
	_debtagsEnabled = enabled;
}

ept::debtags::Debtags* DebtagsPluginContainer::collection() const	
{
	return &(provider()->aptFrontCache().debtags());
}

const std::set<DebtagsPluginContainer::Facet> DebtagsPluginContainer::facets() const
{
	using namespace wibble::operators;

	Vocabulary& tags = provider()->aptFrontCache().vocabulary();
	std::set<Facet> hidden;
	for (set<string>::const_iterator i = _hiddenFacets.begin();
		i != _hiddenFacets.end(); i++)
	{
		Facet f = tags.facetByName(*i);
		if (f.valid())
			hidden |= f;
	}
	return tags.facets() - hidden; 
}




}	// namespace NPlugin

#undef emit
//#include <ept/cache/debtags/tagmap.tcc>
//#include <ept/cache/debtags/vocabulary.tcc>
//#include <ept/cache/apt/index.tcc>
